@@ -12,12 +12,14 @@ of lists and dictionaries.
 import datetime
 import json
 import inspect
-from typing import Any, Union
+from typing import Any, Union, Optional
 from abc import ABC, abstractmethod
 
 
 import hjson
 
+from .shelf import Shelf, Cell # makes import easier
+
 # -------------------------------------
 # Low-level fixtures
 # -------------------------------------
@@ -25,7 +27,7 @@ from collections import UserDict, deque
 
 DICT_TYPES = dict, UserDict
 
-class CustomEncoder(json.JSONEncoder):
+class JSONEncoder(json.JSONEncoder):
     """
     Custom encoder for JSON serialization.
     Used for debugging purposes.
@@ -67,7 +69,7 @@ def json_encode(obj) -> str:
 
     To be used for debugging purposes.
     """
-    return json.dumps(obj, cls=CustomEncoder)
+    return json.dumps(obj, cls=JSONEncoder)
 
 
 def yaml_support():
@@ -99,6 +101,19 @@ def yaml_support():
 
 
 
+def is_elementary_type(t):
+    """
+    Defines an elementary type in Python (str, int, etc.).
+
+    An elementary type is defined as a type that is treated as a value
+    (does not need to be broken down).
+    
+    Do not confuse this with an atomic type, which cannot be subdivided
+    in Python.
+    """
+    return (not hasattr(t, '__dict__') and 
+            not isinstance(t, (list, dict, tuple, set))
+            )
 
 
 
@@ -150,8 +165,6 @@ class SuperDict(dict):
     def __getattr__(self, name:str):
         "Allow dot notation on reading"
         ERR_MSG = "Cannot find attribute '%s'" % name
-        # if name.startswith('_'):
-        #     raise AttributeError(ERR_MSG)
         try:
             return self[name]
         except KeyError:
@@ -212,7 +225,7 @@ class SuperDict(dict):
         particular purpose, except showing what's in structure,
         for string output.
         """
-        return json.dumps(self, cls=CustomEncoder)
+        return json.dumps(self, cls=JSONEncoder)
     
     def to_hjson(self):
         """
@@ -287,7 +300,7 @@ class SuperList(list):
         particular purpose, except showing what's in structure,
         for string output.
         """
-        return json.dumps(self, cls=CustomEncoder)
+        return json.dumps(self, cls=JSONEncoder)
     
     def to_hjson(self):
         """
@@ -396,14 +409,15 @@ def get_dict(obj: Any) -> dict[str, object]:
     raise TypeError(f"Cannot convert of type {obj.__class__.__name__}")
 
 
-
+# These types could be considered as lists but are not
+SPECIAL_ELEMENTARY_TYPES = str, bytes, bytearray
 
 def super_collect(obj:Any) -> Union[SuperDict, SuperList]:
     """
     Factory function:
     Read an object and dispatch it into either a SuperDict or a SuperList
     """
-    if isinstance(obj, (str, bytes, bytearray)):
+    if isinstance(obj, SPECIAL_ELEMENTARY_TYPES):
         raise TypeError(f"Objects of type '{type(obj).__name__}' "
                         "are not accepted (elementary types)")
     try:
@@ -417,6 +431,222 @@ def super_collect(obj:Any) -> Union[SuperDict, SuperList]:
     except TypeError:
         raise TypeError(f"Cannot convert this object of type '{type(obj).__name__}'")
 
+
+
+# -------------------------------------
+# SuperShelf
+# -------------------------------------
+
+class SuperShelf(Shelf):
+    """Shelf subclass that recursively wraps dicts and lists into SuperShelf."""
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        for key, value in self.items():
+            value = self._collect(value)
+            self[key] = value
+
+    def _collect(self, obj:Any) -> "SuperShelf":
+        """
+        Factory function:
+        Read an object and make a SuperShelf
+        """
+        if is_elementary_type(obj):
+            return obj
+        if isinstance(obj, SuperShelf):
+            return obj
+        elif isinstance(obj, Shelf):
+            return SuperShelf(obj)
+        try:
+            list_obj = get_list(obj)
+            return SuperShelf(list_obj)
+        except TypeError:
+            pass
+        try:
+            dict_obj = get_dict(obj)
+            return SuperShelf(dict_obj)
+        except TypeError:
+            pass
+        raise TypeError(f"Cannot collect object of type '{type(obj).__name__}'")
+
+    def append(self, value: Any, label: Optional[str] = None) -> None:
+        """Append value to shelf, recursively wrapping containers."""
+        if isinstance(value, dict) or isinstance(value, list):
+            value = self._collect(value)
+        super().append(value, label=label)
+
+
+    @property
+    def is_fully_collected(self) -> bool:
+        """
+        Validator predicate:
+        Recursively checks whether this SuperShelf contains only:
+        - elementary types
+        - other SuperShelf instances that are themselves fully collected
+        """
+        for cell in self.cells():
+            if not isinstance(cell, (SuperShelf, Cell)):
+                # it's one or the other
+                raise ValueError(f"{cell}: this is not a Supershelf or Cell, but {type(cell)}")
+            elif is_elementary_type(cell.value):
+                pass
+            elif isinstance(cell.value, SuperShelf):
+                if not cell.value.is_fully_collected:
+                    return False
+            else:
+                return False
+        return True
+
+
+
+
+
+    # -------------------------------------
+    # Keys as attributes
+    # -------------------------------------
+    def __getattr__(self, name:str):
+        """
+        Allow dot notation on reading keys, 
+        e.g. foo.bar instead of foo['bar'].
+
+        This is a CONVENIENCE method (for syntactic sugar) and it depends on:
+        - the key being a valid Python identifier.
+        - the attribute not being shadowed by an already existing one in the SuperShelf class.
+        """
+        print("GETATTR called for:", name)
+        ERR_MSG = "Cannot find attribute '%s'" % name
+        try:
+            return self[name]
+        except KeyError:
+            print("Predefined attributes:", list(vars(self).items()))
+            print("Valid:", self.valid_names)
+            print("Object type:", type(self).__name__)
+            raise AttributeError(ERR_MSG)
+
+
+    @property
+    def _reserved_names(self):
+        "Return the set of names reserved by the class or instance (methods, attributes)"
+        return set(dir(type(self))) | set(self.__dict__)
+
+    @property
+    def valid_names(self):
+        """
+        Generate the valid names
+        (the non-shadowed keys that qualify as Python identifiers)
+        """
+        return [
+            item for item in self.keys()
+            if isinstance(item, str)
+            and item.isidentifier()
+            and item not in self._reserved_names
+        ]
+
+
+
+
+    def find(self, key:str, value:Any):
+        "Return the first item where getattr(item, key) == value (but skip elementary types)"
+        for item in self:
+            try:
+                if getattr(item, key) == value:
+                    return item
+            except Exception:
+                continue
+        raise KeyError(f"Key '{key}' not found")
+
+    # -------------------------------------
+    # Output
+    # -------------------------------------
+    @property
+    def _is_strict_list(self) -> bool:
+        "Implementable as list (no labels)"
+        return all(cell.label is None for cell in self.cells())
+
+    @property
+    def _is_strict_dict(self) -> bool:
+        "Implementable as dictionary (all keys are labels)"
+        return all(cell.label is not None for cell in self.cells())
+
+
+
+    def to_json(self, *, indent: int = None) -> str:
+        """
+        Serialize the SuperCollection to a JSON string using the custom JSONEncoder.
+        Each subcollection is optimized as a list, dict, or hybrid depending on addressing mode.
+
+        Parameters:
+            indent (int, optional): Number of spaces for pretty-printing. If None, output is compact.
+
+        Returns:
+            str: JSON string representing the serialized SuperCollection.
+        """
+        structure = [self._serialize_shelf(shelf) for shelf in self]
+        return json.dumps(structure, indent=indent, cls=JSONEncoder)
+
+    def _serialize_shelf(self, item) -> Any:
+        """
+        Serialize a single SuperShelf using the most compact representation:
+        - As a list if all items are unlabelled
+        - As a dict if all items are labelled
+        - As a hybrid list of entries otherwise
+
+        Parameters:
+            shelf: A Shelf-like object supporting iteration and addressing mode predicates.
+
+        Returns:
+            A JSON-serializable structure (list or dict).
+        """
+        # print(f"Item: {item} -> {type(item).__name__}")
+        if is_elementary_type(item):
+            return item
+        elif item._is_strict_list:
+            return [self._serialize_cell(cell) for cell in shelf]
+        elif item._is_strict_dict:
+            return {cell.label: self._serialize_cell(cell) for cell in shelf}
+        else:
+            return [self._serialize_cell(cell) for cell in shelf]
+
+    def _serialize_cell(self, cell) -> Any:
+        """
+        Serialize a single cell:
+        - If labelled: as a singleton dict {label: value}
+        - If unlabelled: as raw value
+        Recursively serializes value if it supports .to_json().
+
+        Parameters:
+            cell: A Cell-like object with .label and .value attributes.
+
+        Returns:
+            A JSON-serializable value or singleton dict.
+        """
+        value = cell.value
+        if hasattr(value, "to_json") and callable(value.to_json):
+            value = value.to_json()
+        return {cell.label: value} if cell.label is not None else value
+
+    def to_hjson(self):
+        """
+        Convert to hjson.
+
+        It does not have any claim of fitness for any
+        particular purpose, except showing what's in structure,
+        for string output.
+        """
+        obj = json.loads(self.to_json())
+        return hjson.dumps(obj)
+
+    def __str__(self):
+        "Print to hjson"
+        return self.to_hjson()
+
+    def __rich__(self):
+        "Print (for rich)"
+        r = [f"[bold red]{self.__class__.__name__}:[/]"]
+        r.append(self.to_hjson())
+        return("\n".join(r))      
+
+
 # -------------------------------------
 # Super Collection
 # -------------------------------------
@@ -431,10 +661,6 @@ class SuperCollection(ABC):
         return super_collect(obj)
     
     @abstractmethod
-    def __post_init__(self):
-        "Recursively transform collection"
-    
-
     def to_json(self):
         """
         Convert to json.
@@ -447,9 +673,20 @@ class SuperCollection(ABC):
         """
         pass
 
+    @abstractmethod
+    def to_hjson(self):
+        """
+        Convert to hjson.
+
+        It does not have any claim of fitness for any
+        particular purpose, except showing what's in structure,
+        for string output.
+        """
+        pass
+
     @abstractmethod
     def __str__(self):
-        "Print the object (current convention is hjson, no json)"
+        "Print (to hjson, in principle)"
         pass
     
     @abstractmethod
@@ -458,4 +695,5 @@ class SuperCollection(ABC):
         pass
 
 SuperCollection.register(SuperList)
-SuperCollection.register(SuperDict)
\ No newline at end of file
+SuperCollection.register(SuperDict)
+SuperCollection.register(SuperShelf)
